 /*
  * UAE - The Un*x Amiga Emulator
  *
  * Sleeping for *nix systems
  *
  * Copyright 2003-2005 Richard Drummond
  */

//#if 1 //ndef _WIN32

#include "sysconfig.h"
#include "sysdeps.h"

#include "uae.h"
#include "custom.h"
#include "options.h"
#include "events.h"
#include "hrtimer.h"
#include "sleep.h"


/* Define me to always sleep no matter what the latency
 * of the sleep function */
//#define SLEEP_DONT_BUSY_WAIT

/* Busy sleep threshhold in ms. If busy-waiting is enabled, sleeps for periods
 * shorter than this will always be done with a busy loop
 *
 * We should probably determine the threshhold at run-time, but a constant value
 * works well enough
 */
#define SLEEP_BUSY_THRESHOLD	 10


#define DO_REALTIME 1


#define RTC_FREQ 1024

#include <stdbool.h>
#include <pthread.h>
#include <linux/rtc.h>
#include <sys/ioctl.h>
#include <sched.h>
#include <sys/types.h>
#include <signal.h>

static int rtc_fd;

#define GIVERTCAP_REALTIME 0


static pid_t other_pid;  
static bool watchdog_thread_started=false;
static bool isprocessing=false;
int lastactive=0;
static pthread_t watchdog_thread;

static void *watchdog(void *arg){
  struct sched_param par;

  system("/usr/bin/givertcap");

  par.sched_priority = sched_get_priority_max(SCHED_FIFO);
  if(sched_setscheduler(0,SCHED_FIFO,&par)==-1){
    fprintf(stderr,"VSTSERVANT/PROCESS_watchdog: Unable to set SCHED_FIFO realtime priority for the watchdog thread. No watchdog.\n");
    goto exit;
  }

  watchdog_thread_started=true;

  static int num=0;

  for(;;){
    int last=lastactive;
    sleep(1);
    num++;
    //if(client->endwatchdog==true) goto exit;
    if(last==lastactive || num==400){
      struct sched_param par;
      fprintf(stderr,"setting to non-realtime for 2 seconds.\n");
      //kill(client->processthread_pid,SIGKILL);
      par.sched_priority = 0;
      if(sched_setscheduler(other_pid,SCHED_OTHER,&par)==-1){
	fprintf(stderr,"Damn!\n");
	kill(other_pid,SIGKILL);
      }
      sleep(2);
      if(1 || num<400){
	par.sched_priority = sched_get_priority_min(SCHED_FIFO);
	fprintf(stderr,"Setting back to realtime priority.\n");
	if(sched_setscheduler(other_pid,SCHED_FIFO,&par)==-1){
	  fprintf(stderr,"Could not set back...\n");
	}
      }
      //goto exit;
    }
  }
 exit:
  fprintf(stderr,"Watchdog exiting\n");
  return NULL;
}




void rtc_setup(void)
{

  other_pid=getpid();

#if DO_REALTIME
  uid_t uid=getuid();
  system("/usr/bin/givertcap");
  struct sched_param par;
  par.sched_priority = sched_get_priority_min(SCHED_FIFO);
  if(pthread_create(&watchdog_thread,NULL,watchdog,NULL)!=0){
    fprintf(stderr,"Could not create watchdog. Not running reatlime\n");
  }else{
#if 1
    if(sched_setscheduler(0,SCHED_FIFO,&par)==-1){
      printf("urk\n");
    }
#endif
  }
  setuid(uid);
#endif


#ifdef USE_RTC
    if ((rtc_fd = open("/dev/rtc", O_RDONLY)) < 0) {
      printf("Cant open /dev/rtc because %s", strerror(errno));
    } else {
      unsigned int irqp = RTC_FREQ;
      if (ioctl(rtc_fd, RTC_IRQP_SET, irqp) < 0) {
	printf("Could not set proper irq frequency %u %s\n", irqp,
	       strerror(errno));
	close(rtc_fd);
	rtc_fd = -1;
      } else if (ioctl(rtc_fd, RTC_PIE_ON, 0) < 0) {
	/* variable only by the root */
	printf("Some Pie On (?) error with /dev/rtc\n");
	close(rtc_fd);
	rtc_fd = -1;
      } else
	printf("Using /dev/rtc as timer.\n");
    }
#endif
}

#ifdef USE_RTC
void rtc_sleep(int time){
  unsigned long rtc_ts;

#if GIVERTCAP_REALTIME
  struct sched_param par;
  par.sched_priority = (sched_get_priority_max(SCHED_FIFO) + sched_get_priority_min(SCHED_FIFO) ) / 2;
  if(sched_setscheduler(0,SCHED_FIFO,&par)==-1){
    printf("urk\n");
  }
#endif

  time*=RTC_FREQ;
  time/=1024;

  while(time>0) {
    lastactive++;
    if (read (rtc_fd, &rtc_ts, sizeof(rtc_ts)) <= 0)
      printf("Could not read /dev/rtc gakk! %s",strerror(errno));
    time--;
  }

#if GIVERTCAP_REALTIME
  par.sched_priority = 0;
  if(sched_setscheduler(0,SCHED_OTHER,&par)==-1){
    printf("urk2\n");
  }
#endif
}
#endif


#include <sys/poll.h>
struct pollfd das_poll[1];


void sleep_millis2(int time){
  unsigned long rtc_ts;

#if GIVERTCAP_REALTIME
  struct sched_param par;
  par.sched_priority = (sched_get_priority_max(SCHED_FIFO) + sched_get_priority_min(SCHED_FIFO) ) / 2;
  if(sched_setscheduler(0,SCHED_FIFO,&par)==-1){
    printf("urk\n");
  }
#endif
  lastactive++;
#if 1
  if(0){
    static int w=0;
    w++;
   if(w>10){
     sleep(1);
     w=0;
   }
  }
  //usleep(2);
  sched_yield();
#endif

  das_poll[0].fd=-1;
  das_poll[0].events = POLLIN|POLLERR|POLLHUP|POLLNVAL;
  //printf("1\n");
  poll(das_poll,1,1);
  //printf("2\n");

  //  if (read (rtc_fd, &rtc_ts, sizeof(rtc_ts)) <= 0)
  //  printf("Could not read /dev/rtc gakk2! %s",strerror(errno));

  //printf("3\n");
#if GIVERTCAP_REALTIME
  par.sched_priority = 0;
  if(sched_setscheduler(0,SCHED_OTHER,&par)==-1){
    printf("urk2\n");
  }
#endif
}



/*
 * Sleep for ms milliseconds either using an appropriate sleep routine on the
 * target system or, if the target's routine has too much latency to
 * accurately sleep for that period, then busy wait.
 *
 * Busy-waiting requires a high-resolution timer for accuracy. We use
 * the machine-dependent read_processor_time ()
 */
void sleep_millis (int ms)
{
    frame_time_t start = uae_gethrtime ();
    frame_time_t sleep_time;

#ifndef SLEEP_DONT_BUSY_WAIT
    //#if 0
    if (!currprefs.dont_busy_wait && ms < SLEEP_BUSY_THRESHOLD) {
	/* Typical sleep routines can't sleep for less than 10ms. If we want
	 * to sleep for a period shorter than the threshold, we'll have to busy wait . . .
	 */
	frame_time_t end = start + ms * uae_gethrtimebase () / 1000;
	frame_time_t v;

        do {
	    v = uae_gethrtime ();
	} while (v < end && v > -end);
    } else
#endif
	uae_msleep (ms);

    sleep_time = uae_gethrtime () - start;
    idletime += sleep_time;
}



/*
 * Sleep for ms milliseconds if and only busy-waiting would not be required
 * to do so.
 */
void sleep_millis_busy (int ms)
{
#ifndef SLEEP_DONT_BUSY_WAIT
    if (currprefs.dont_busy_wait || ms >= SLEEP_BUSY_THRESHOLD)
#endif
	sleep_millis (ms);
}

/*
 * Measure how long it takes to do a ms millisecond sleep. Timing is performed
 * with a machine-specific high-resolution timer.
 */
static int do_sleep_test (int ms)
{
    frame_time_t t;
    frame_time_t t2;

    t = uae_gethrtime ();
    uae_msleep (ms);
    t2 = uae_gethrtime () - t;

    if (t2 < 0)
	t2 = -t2;

    return t2;
}

/*
 * Test the target system's sleep routine to decide whether we should busy wait
 * by default for sleeps shorter in duration than SLEEP_BUSY_THRESHOLD
 */
void sleep_test (void)
{
  float result;

    currprefs.dont_busy_wait = 1;

#ifndef SLEEP_DONT_BUSY_WAIT
    //#if 0
    {
	uae_u64 total = 0;
	float result;
	int num_tests;
	int i;

	write_log ("Testing system sleep function"); flush_log ();

	/* Do a few tests to get a rough idea how fast we can do it */
	num_tests = 16;

	for (i=0; i < num_tests; i++)
	    total += do_sleep_test (1);

	/* How many for 2 seconds worth of tests . . . */
	num_tests = 2 * uae_gethrtimebase () * num_tests / total;
	total = 0;

	/* Now the test proper */
	for (i = 0; i < num_tests; i++) {
	    total += do_sleep_test (1);

	    if (i - (i % 100) == i) {
		write_log (".");
		flush_log ();
	    }
	}

	result = (float)(1000 * total / uae_gethrtimebase ()) / (float)num_tests;
	write_log ("\nAverage duration of a 1ms sleep: %f ms\n", result);

	if (result > SLEEP_BUSY_THRESHOLD) {
	    currprefs.dont_busy_wait = 0;
	    write_log ("Enabling busy-waiting for sub-%dms sleeps\n", SLEEP_BUSY_THRESHOLD);
	}
    }

#endif
}

//#endif /* !_WIN32 */
