

// Made by Kjetil S. Matheussen, k.s.matheussen@notam02.no
//
// I compile it like this: sc uaecamd.c link optimize ignore=51 nostackcheck debug=line LIB LIB:debug.lib strict ansi

#include <dos/dostags.h>
#include <dos/dos.h>
#include <stdio.h>
#include <stdbool.h>
#include <stdlib.h>
#include <string.h>
#include <proto/exec.h>
#include <clib/camd_protos.h>
#include <midi/camd.h>
#include <pragmas/camd_pragmas.h>
#include <exec/memory.h>

#include <proto/dos.h>

#define POLL_FREQUENCY 10024 // times a second.


static ULONG (*calltrap1)(ULONG,ULONG,ULONG) = (ULONG (*)())0xF0FF60;
static ULONG (*calltrap2)(ULONG,ULONG) = (ULONG (*)())0xF0FF60;

static ULONG camd_PutMidi(ULONG device,ULONG data) //1,ULONG data2,ULONG data3)
{
  (*calltrap1) (90,device,data);
  return 0;
}
static ULONG camd_GetMidi(ULONG device)
{
  return (*calltrap2) (91,device);
}


static struct Library *CamdBase=NULL;

static struct MidiNode *midinode=NULL;
static struct MidiLink *inlink=NULL;
static struct MidiLink *outlink=NULL;


static char *inlinkname;
static char *outlinkname;

static char *midinodename;

static ULONG dev_num;

static struct Hook hook;
static bool running=true;
static bool putproc_running=true;

static struct MidiNode *CreateMidiB(struct Library *CamdBase,Tag tag, ...)
{	return CreateMidiA((struct TagItem *)&tag );
}

static struct MidiLink *AddMidiLinkB(struct Library *CamdBase,struct MidiNode *mi, LONG type, Tag tag, ...)
{	return AddMidiLinkA(mi, type, (struct TagItem *)&tag );
}

static ULONG __asm __saveds RecvHook(register __a0 const struct Hook *hook,
                                       register __a2 const struct MidiLink *link,
                                       register __a1 const MidiMsg *msg,
                                       register __d0 const long sysexlen,
                                       register __a3 const void *sysexdata
){
//	kprintf("New. msg: %lx\n",msg->mm_Msg);
  if(outlink!=NULL){
    if(msg->mm_Status!=0xf0){
      camd_PutMidi(dev_num,msg->l[0]>>8);
    }
  }
  return 0;
}


static void __saveds PutProc(void){
  struct timerequest *TimerIO;
  struct MsgPort  *TimerMP;
  MidiMsg msg;
  TimerMP=CreateMsgPort();
  TimerIO=(struct timerequest *)AllocMem(sizeof(struct timerequest),MEMF_ANY|MEMF_CLEAR|MEMF_PUBLIC);
  TimerIO->tr_node.io_Message.mn_Node.ln_Type=NT_MESSAGE;
  TimerIO->tr_node.io_Message.mn_ReplyPort=TimerMP;
  TimerIO->tr_node.io_Message.mn_Length=sizeof(struct timerequest);
  OpenDevice(
	     TIMERNAME,UNIT_ECLOCK,(struct IORequest *)TimerIO,0L
	     );
  while(running==true){
    TimerIO->tr_node.io_Command=TR_ADDREQUEST;
    TimerIO->tr_time.tv_secs=0;
    TimerIO->tr_time.tv_micro=1000000/POLL_FREQUENCY;
    DoIO((struct IORequest *)TimerIO);
    msg.l[0]=camd_GetMidi(dev_num);
    if(msg.l[0]>0){
      PutMidiMsg(inlink,&msg);
    }
  }
  CloseDevice((struct IORequest *)TimerIO);
  FreeMem(TimerIO,sizeof(struct timerequest));
  DeleteMsgPort(TimerMP);
  putproc_running=false;
}


int main(int argc,char **argv){
	int ret= -1;
	char linkname[500];

	/***************** Init ***************************************/

	//printf("argc %d, arg0: %s, arg1: %s",argc,argv[0],argv[1]);
	if(argc!=2 && argc!=3){
		printf(
		       "Usage: %s <device number> [linkname]\n",
			argv[0]
		);
		printf(
		       "Default value for linkname is uae.\n"
			"Sysex messages are unfortunately not handled.\n"
		);
		return 0;
	}

	dev_num=atoi(argv[1]);

	if(argc==2)
	  sprintf(linkname,"uae");
	else
	  sprintf(linkname,argv[2]);

	// Setting up camd

	CamdBase=OpenLibrary("camd.library",40L);
	if(CamdBase==NULL){
	  printf("Could not open camd.library at least V0.40.\n");
	  goto exit;
	}

	midinodename=malloc(500);
	sprintf(midinodename,"uae %d",dev_num);

	inlinkname=malloc(500);
	sprintf(inlinkname,"%s.in.%d",linkname,dev_num);

	outlinkname=malloc(500);
	sprintf(outlinkname,"%s.out.%d",linkname,dev_num);

	hook.h_Entry=(ULONG(*)())RecvHook;

	midinode=CreateMidiB(CamdBase,
		MIDI_Name,midinodename,
		MIDI_MsgQueue, 10000,
		MIDI_RecvHook,&hook,
		TAG_END
	);
	if(midinode==NULL){
		printf("No midinode, new\n");
		goto exit;
	}


	inlink=AddMidiLinkB(CamdBase,
			    midinode,MLTYPE_Sender,
			    MLINK_Name, inlinkname,
			    MLINK_Location, inlinkname,
			    TAG_END
			    );

	if(inlink==NULL){
		printf("No inmidilink, new\n");
		goto exit;
	}

	outlink=AddMidiLinkB(CamdBase,
			     midinode,MLTYPE_Receiver,
			     MLINK_Name, outlinkname,
			     MLINK_Location, outlinkname,
			     TAG_END
	);

	if(outlink==NULL){
		printf("No outmidilink, new\n");
		goto exit;
	}


	
	//camd_PutMidi(0,0x906060);


	// Create poll process

	CreateNewProcTags(
			  NP_Entry,PutProc,
			  NP_Name,"camd_uae_putproc",
			  NP_Priority,36,
			  TAG_END
			  );


/**************************************************************
      Waiting for CTRL_C
**************************************************************/
	Wait(SIGBREAKF_CTRL_C);

	ret=0;

/******* exiting poll process **********************************/
	running=false;
	while(putproc_running==true){
	  Delay(1);
	}

exit:

	if(midinode!=NULL){
		DeleteMidi(midinode);
	}
	if(CamdBase!=NULL){
		CloseLibrary(CamdBase);
	}

	return ret;
}

